@@ -123,6 +123,11 @@ fn main() {
             src_dir: "vendored_parsers/tree-sitter-dart-src",
             extra_files: vec!["scanner.c"],
         },
+        TreeSitterParser {
+            name: "tree-sitter-devicetree",
+            src_dir: "vendored_parsers/tree-sitter-devicetree-src",
+            extra_files: vec![],
+        },
         TreeSitterParser {
             name: "tree-sitter-elisp",
             src_dir: "vendored_parsers/tree-sitter-elisp-src",
@@ -148,6 +153,11 @@ fn main() {
             src_dir: "vendored_parsers/tree-sitter-erlang-src",
             extra_files: vec![],
         },
+        TreeSitterParser {
+            name: "tree-sitter-f-sharp",
+            src_dir: "vendored_parsers/tree-sitter-f-sharp-src",
+            extra_files: vec!["scanner.c"],
+        },
         TreeSitterParser {
             name: "tree-sitter-gleam",
             src_dir: "vendored_parsers/tree-sitter-gleam-src",
