@@ -1,8 +1,9 @@
 //! A fallback "parser" for plain text.
 
 use lazy_static::lazy_static;
-use line_numbers::LinePositions;
+use line_numbers::{LinePositions, SingleLineSpan};
 use regex::Regex;
+use std::hash::Hash;
 
 use crate::words::split_words;
 use crate::{
@@ -73,24 +74,66 @@ fn merge_novel<'a>(
     res
 }
 
+#[derive(Debug, Clone)]
+struct StringIgnoringNewline<'a>(&'a str);
+
+impl PartialEq for StringIgnoringNewline<'_> {
+    fn eq(&self, other: &Self) -> bool {
+        let mut s = self.0;
+        if s.ends_with('\n') {
+            s = &s[..s.len() - 1];
+        }
+
+        let mut other_s = other.0;
+        if other_s.ends_with('\n') {
+            other_s = &other_s[..other_s.len() - 1];
+        }
+
+        s == other_s
+    }
+}
+
+impl Eq for StringIgnoringNewline<'_> {}
+
+impl Hash for StringIgnoringNewline<'_> {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        let mut s = self.0;
+        if s.ends_with('\n') {
+            s = &s[..s.len() - 1];
+        }
+
+        s.hash(state);
+    }
+}
+
 fn changed_parts<'a>(
     src: &'a str,
     opposite_src: &'a str,
 ) -> Vec<(TextChangeKind, Vec<&'a str>, Vec<&'a str>)> {
-    let src_lines = split_lines_keep_newline(src);
-    let opposite_src_lines = split_lines_keep_newline(opposite_src);
+    let src_lines = split_lines_keep_newline(src)
+        .into_iter()
+        .map(StringIgnoringNewline)
+        .collect::<Vec<_>>();
+    let opposite_src_lines = split_lines_keep_newline(opposite_src)
+        .into_iter()
+        .map(StringIgnoringNewline)
+        .collect::<Vec<_>>();
 
     let mut res: Vec<(TextChangeKind, Vec<&'a str>, Vec<&'a str>)> = vec![];
     for diff_res in myers_diff::slice_unique_by_hash(&src_lines, &opposite_src_lines) {
         match diff_res {
             myers_diff::DiffResult::Left(line) => {
-                res.push((TextChangeKind::Novel, vec![line], vec![]));
+                res.push((TextChangeKind::Novel, vec![line.0], vec![]));
             }
             myers_diff::DiffResult::Both(line, opposite_line) => {
-                res.push((TextChangeKind::Unchanged, vec![line], vec![opposite_line]));
+                res.push((
+                    TextChangeKind::Unchanged,
+                    vec![line.0],
+                    vec![opposite_line.0],
+                ));
             }
             myers_diff::DiffResult::Right(opposite_line) => {
-                res.push((TextChangeKind::Novel, vec![], vec![opposite_line]));
+                res.push((TextChangeKind::Novel, vec![], vec![opposite_line.0]));
             }
         }
     }
@@ -118,9 +161,12 @@ pub(crate) fn change_positions(lhs_src: &str, rhs_src: &str) -> Vec<MatchedPos>
     let mut rhs_offset = 0;
 
     let mut mps = vec![];
+
+    let mut seen_unchanged = false;
     for (kind, lhs_lines, rhs_lines) in changed_parts(lhs_src, rhs_src) {
         match kind {
             TextChangeKind::Unchanged => {
+                seen_unchanged = true;
                 for (lhs_line, rhs_line) in lhs_lines.iter().zip(rhs_lines) {
                     let lhs_pos =
                         lhs_lp.from_region(lhs_offset, lhs_offset + line_len_in_bytes(lhs_line));
@@ -153,12 +199,14 @@ pub(crate) fn change_positions(lhs_src: &str, rhs_src: &str) -> Vec<MatchedPos>
                 // individual words.
                 if lhs_words.len() > MAX_WORDS_IN_LINE || rhs_words.len() > MAX_WORDS_IN_LINE {
                     for lhs_pos in lhs_lp.from_region(lhs_offset, lhs_offset + lhs_part.len()) {
-                        mps.push(MatchedPos {
-                            kind: MatchKind::NovelWord {
-                                highlight: TokenKind::Atom(AtomKind::Normal),
-                            },
-                            pos: lhs_pos,
-                        });
+                        if lhs_pos.start_col != lhs_pos.end_col {
+                            mps.push(MatchedPos {
+                                kind: MatchKind::NovelWord {
+                                    highlight: TokenKind::Atom(AtomKind::Normal),
+                                },
+                                pos: lhs_pos,
+                            });
+                        }
                     }
 
                     lhs_offset += lhs_part.len();
@@ -189,7 +237,7 @@ pub(crate) fn change_positions(lhs_src: &str, rhs_src: &str) -> Vec<MatchedPos>
                                     rhs_lp.from_region(rhs_offset, rhs_offset + rhs_word.len());
 
                                 mps.push(MatchedPos {
-                                    kind: MatchKind::NovelLinePart {
+                                    kind: MatchKind::UnchangedPartOfNovelItem {
                                         highlight: TokenKind::Atom(AtomKind::Normal),
                                         self_pos: lhs_pos[0],
                                         opposite_pos: rhs_pos,
@@ -210,6 +258,33 @@ pub(crate) fn change_positions(lhs_src: &str, rhs_src: &str) -> Vec<MatchedPos>
         }
     }
 
+    // If there are no unchanged items, insert a dummy item at the
+    // beginning of both files with a width of zero. This gives
+    // display something to use when aligning.
+    if !seen_unchanged {
+        let lhs_pos = SingleLineSpan {
+            line: 0.into(),
+            start_col: 0,
+            end_col: 0,
+        };
+        let rhs_pos = SingleLineSpan {
+            line: 0.into(),
+            start_col: 0,
+            end_col: 0,
+        };
+        mps.insert(
+            0,
+            MatchedPos {
+                kind: MatchKind::UnchangedToken {
+                    highlight: TokenKind::Atom(AtomKind::Normal),
+                    self_pos: vec![lhs_pos],
+                    opposite_pos: vec![rhs_pos],
+                },
+                pos: lhs_pos,
+            },
+        );
+    }
+
     mps
 }
 
@@ -248,8 +323,9 @@ mod tests {
         // Even though the word exists on both sides, it should still
         // be treated as a change. We're doing a line-based diff and
         // the lines are different.
-        let positions = change_positions("foo", " foo");
-        assert!(positions[0].kind.is_novel());
+        let mut positions = change_positions("foo", " foo");
+        let last_pos = positions.pop().unwrap();
+        assert!(last_pos.kind.is_novel());
     }
 
     #[test]
@@ -270,17 +346,17 @@ mod tests {
 
     #[test]
     fn test_novel_lhs_trailing_newlines() {
-        let positions = change_positions("foo\n", "");
+        let mut positions = change_positions("foo\n", "");
 
-        assert_eq!(positions.len(), 2);
-        assert!(positions[0].kind.is_novel());
+        let last_pos = positions.pop().unwrap();
+        assert!(last_pos.kind.is_novel());
     }
 
     #[test]
     fn test_positions_novel_lhs() {
-        let positions = change_positions("foo", "");
+        let mut positions = change_positions("foo", "");
 
-        assert_eq!(positions.len(), 1);
-        assert!(positions[0].kind.is_novel());
+        let last_pos = positions.pop().unwrap();
+        assert!(last_pos.kind.is_novel());
     }
 }
