@@ -598,15 +598,33 @@ pub(crate) enum MatchKind {
     },
     /// A novel token in an AST diff.
     Novel { highlight: TokenKind },
-    /// When doing a textual line-based diff, the part of a novel line
-    /// that exists on both sides.
-    NovelLinePart {
+    /// When we have a novel item, we often want to highlight novel
+    /// words more prominently. UnchangedPartOfNovelItem represents
+    /// the parts that don't get this special highlighting.
+    ///
+    /// For example, line-based diffs we want to highlight `a` and `b`
+    /// differently to `foo` here.
+    ///
+    /// foo a
+    /// foo b
+    ///
+    /// Whereas for syntactic diffs, we want to do the same thing for
+    /// strings and comments.
+    ///
+    /// "foo a"
+    /// "foo b"
+    ///
+    /// The whole string is a distinct value, but the `a` and `b` are
+    /// the most interesting parts.
+    UnchangedPartOfNovelItem {
         highlight: TokenKind,
         self_pos: SingleLineSpan,
         opposite_pos: Vec<SingleLineSpan>,
     },
-    /// When doing a textual line-based diff, the part of a novel line
-    /// that is actually novel. E.g. the newly added word on a line.
+    /// The novel part of the novel item. For line-based diffs, this
+    /// is the words that are unique to this line.
+    ///
+    /// See the discussion in `UnchangedPartOfNovelItem`.
     NovelWord { highlight: TokenKind },
     /// A syntactic token that was ignored by the AST diff (e.g. when
     /// ignoring comments for diffing).
@@ -617,7 +635,9 @@ impl MatchKind {
     pub(crate) fn is_novel(&self) -> bool {
         matches!(
             self,
-            MatchKind::Novel { .. } | MatchKind::NovelWord { .. } | MatchKind::NovelLinePart { .. }
+            MatchKind::Novel { .. }
+                | MatchKind::NovelWord { .. }
+                | MatchKind::UnchangedPartOfNovelItem { .. }
         )
     }
 }
@@ -700,7 +720,7 @@ fn split_atom_words(
                 );
 
                 mps.push(MatchedPos {
-                    kind: MatchKind::NovelLinePart {
+                    kind: MatchKind::UnchangedPartOfNovelItem {
                         highlight: TokenKind::Atom(kind),
                         self_pos: word_pos,
                         opposite_pos: opposite_word_pos,
@@ -875,7 +895,37 @@ pub(crate) fn change_positions<'a>(
     change_map: &ChangeMap<'a>,
 ) -> Vec<MatchedPos> {
     let mut positions = Vec::new();
-    change_positions_(nodes, change_map, &mut positions);
+    let mut seen_unchanged = false;
+
+    change_positions_(nodes, change_map, &mut positions, &mut seen_unchanged);
+
+    // If there are no unchanged items, insert a dummy item at the
+    // beginning of both files with a width of zero. This gives
+    // display something to use when aligning.
+    if !seen_unchanged {
+        let lhs_pos = SingleLineSpan {
+            line: 0.into(),
+            start_col: 0,
+            end_col: 0,
+        };
+        let rhs_pos = SingleLineSpan {
+            line: 0.into(),
+            start_col: 0,
+            end_col: 0,
+        };
+        positions.insert(
+            0,
+            MatchedPos {
+                kind: MatchKind::UnchangedToken {
+                    highlight: TokenKind::Atom(AtomKind::Normal),
+                    self_pos: vec![lhs_pos],
+                    opposite_pos: vec![rhs_pos],
+                },
+                pos: lhs_pos,
+            },
+        );
+    }
+
     positions
 }
 
@@ -883,12 +933,17 @@ fn change_positions_<'a>(
     nodes: &[&'a Syntax<'a>],
     change_map: &ChangeMap<'a>,
     positions: &mut Vec<MatchedPos>,
+    seen_unchanged: &mut bool,
 ) {
     for node in nodes {
         let change = change_map
             .get(node)
             .unwrap_or_else(|| panic!("Should have changes set in all nodes: {:#?}", node));
 
+        if matches!(change, ChangeKind::Unchanged(_)) {
+            *seen_unchanged = true;
+        }
+
         match node {
             List {
                 open_position,
@@ -903,7 +958,7 @@ fn change_positions_<'a>(
                     false,
                 ));
 
-                change_positions_(children, change_map, positions);
+                change_positions_(children, change_map, positions, seen_unchanged);
 
                 positions.extend(MatchedPos::new(
                     change,
@@ -1149,7 +1204,7 @@ mod tests {
             res,
             vec![
                 MatchedPos {
-                    kind: MatchKind::NovelLinePart {
+                    kind: MatchKind::UnchangedPartOfNovelItem {
                         highlight: TokenKind::Atom(AtomKind::Comment),
                         self_pos: SingleLineSpan {
                             line: 0.into(),
@@ -1169,7 +1224,7 @@ mod tests {
                     }
                 },
                 MatchedPos {
-                    kind: MatchKind::NovelLinePart {
+                    kind: MatchKind::UnchangedPartOfNovelItem {
                         highlight: TokenKind::Atom(AtomKind::Comment),
                         self_pos: SingleLineSpan {
                             line: 0.into(),
@@ -1189,7 +1244,7 @@ mod tests {
                     }
                 },
                 MatchedPos {
-                    kind: MatchKind::NovelLinePart {
+                    kind: MatchKind::UnchangedPartOfNovelItem {
                         highlight: TokenKind::Atom(AtomKind::Comment),
                         self_pos: SingleLineSpan {
                             line: 0.into(),
@@ -1209,7 +1264,7 @@ mod tests {
                     }
                 },
                 MatchedPos {
-                    kind: MatchKind::NovelLinePart {
+                    kind: MatchKind::UnchangedPartOfNovelItem {
                         highlight: TokenKind::Atom(AtomKind::Comment),
                         self_pos: SingleLineSpan {
                             line: 0.into(),
@@ -1229,7 +1284,7 @@ mod tests {
                     }
                 },
                 MatchedPos {
-                    kind: MatchKind::NovelLinePart {
+                    kind: MatchKind::UnchangedPartOfNovelItem {
                         highlight: TokenKind::Atom(AtomKind::Comment),
                         self_pos: SingleLineSpan {
                             line: 0.into(),
