@@ -1,3 +1,15 @@
+/**
+ * @file JavaScript grammar for tree-sitter
+ * @author Max Brunsfeld <maxbrunsfeld@gmail.com>
+ * @license MIT
+ */
+
+/* eslint-disable arrow-parens */
+/* eslint-disable camelcase */
+/* eslint-disable-next-line spaced-comment */
+/// <reference types="tree-sitter-cli/dsl" />
+// @ts-check
+
 module.exports = grammar({
   name: 'javascript',
 
@@ -5,11 +17,18 @@ module.exports = grammar({
     $._automatic_semicolon,
     $._template_chars,
     $._ternary_qmark,
+    $.html_comment,
+    '||',
+    // We use escape sequence and regex pattern to tell the scanner if we're currently inside a string or template string, in which case
+    // it should NOT parse html comments.
+    $.escape_sequence,
+    $.regex_pattern,
   ],
 
   extras: $ => [
     $.comment,
-    /[\s\p{Zs}\uFEFF\u2060\u200B]/,
+    $.html_comment,
+    /[\s\p{Zs}\uFEFF\u2028\u2029\u2060\u200B]/,
   ],
 
   supertypes: $ => [
@@ -58,7 +77,7 @@ module.exports = grammar({
       'logical_or',
       'ternary',
       $.sequence_expression,
-      $.arrow_function
+      $.arrow_function,
     ],
     ['assign', $.primary_expression],
     ['member', 'new', 'call', $.expression],
@@ -66,6 +85,7 @@ module.exports = grammar({
     [$.primary_expression, $.statement_block, 'object'],
     [$.import_statement, $.import],
     [$.export_statement, $.primary_expression],
+    [$.lexical_declaration, $.primary_expression],
   ],
 
   conflicts: $ => [
@@ -90,10 +110,10 @@ module.exports = grammar({
   rules: {
     program: $ => seq(
       optional($.hash_bang_line),
-      repeat($.statement)
+      repeat($.statement),
     ),
 
-    hash_bang_line: $ => /#!.*/,
+    hash_bang_line: _ => /#!.*/,
 
     //
     // Export declarations
@@ -121,31 +141,31 @@ module.exports = grammar({
               field('declaration', $.declaration),
               seq(
                 field('value', $.expression),
-                $._semicolon
-              )
-            )
-          )
-        )
-      )
+                $._semicolon,
+              ),
+            ),
+          ),
+        ),
+      ),
     ),
 
     namespace_export: $ => seq(
-      '*', 'as', $._module_export_name
+      '*', 'as', $._module_export_name,
     ),
 
     export_clause: $ => seq(
       '{',
       commaSep($.export_specifier),
       optional(','),
-      '}'
+      '}',
     ),
 
     export_specifier: $ => seq(
       field('name', $._module_export_name),
       optional(seq(
         'as',
-        field('alias', $._module_export_name)
-      ))
+        field('alias', $._module_export_name),
+      )),
     ),
 
     _module_export_name: $ => choice(
@@ -158,22 +178,23 @@ module.exports = grammar({
       $.generator_function_declaration,
       $.class_declaration,
       $.lexical_declaration,
-      $.variable_declaration
+      $.variable_declaration,
     ),
 
     //
     // Import declarations
     //
 
-    import: $ => token('import'),
+    import: _ => token('import'),
 
     import_statement: $ => seq(
       'import',
       choice(
         seq($.import_clause, $._from_clause),
-        field('source', $.string)
+        field('source', $.string),
       ),
-      $._semicolon
+      optional($.import_attribute),
+      $._semicolon,
     ),
 
     import_clause: $ => choice(
@@ -185,25 +206,25 @@ module.exports = grammar({
           ',',
           choice(
             $.namespace_import,
-            $.named_imports
-          )
-        ))
-      )
+            $.named_imports,
+          ),
+        )),
+      ),
     ),
 
     _from_clause: $ => seq(
-      "from", field('source', $.string)
+      'from', field('source', $.string),
     ),
 
     namespace_import: $ => seq(
-      "*", "as", $.identifier
+      '*', 'as', $.identifier,
     ),
 
     named_imports: $ => seq(
       '{',
       commaSep($.import_specifier),
       optional(','),
-      '}'
+      '}',
     ),
 
     import_specifier: $ => choice(
@@ -211,10 +232,12 @@ module.exports = grammar({
       seq(
         field('name', $._module_export_name),
         'as',
-        field('alias', $.identifier)
+        field('alias', $.identifier),
       ),
     ),
 
+    import_attribute: $ => seq('with', $.object),
+
     //
     // Statements
     //
@@ -241,36 +264,36 @@ module.exports = grammar({
       $.return_statement,
       $.throw_statement,
       $.empty_statement,
-      $.labeled_statement
+      $.labeled_statement,
     ),
 
     expression_statement: $ => seq(
       $._expressions,
-      $._semicolon
+      $._semicolon,
     ),
 
     variable_declaration: $ => seq(
       'var',
       commaSep1($.variable_declarator),
-      $._semicolon
+      $._semicolon,
     ),
 
     lexical_declaration: $ => seq(
       field('kind', choice('let', 'const')),
       commaSep1($.variable_declarator),
-      $._semicolon
+      $._semicolon,
     ),
 
     variable_declarator: $ => seq(
       field('name', choice($.identifier, $._destructuring_pattern)),
-      optional($._initializer)
+      optional($._initializer),
     ),
 
     statement_block: $ => prec.right(seq(
       '{',
       repeat($.statement),
       '}',
-      optional($._automatic_semicolon)
+      optional($._automatic_semicolon),
     )),
 
     else_clause: $ => seq('else', $.statement),
@@ -279,13 +302,13 @@ module.exports = grammar({
       'if',
       field('condition', $.parenthesized_expression),
       field('consequence', $.statement),
-      optional(field('alternative', $.else_clause))
+      optional(field('alternative', $.else_clause)),
     )),
 
     switch_statement: $ => seq(
       'switch',
       field('value', $.parenthesized_expression),
-      field('body', $.switch_body)
+      field('body', $.switch_body),
     ),
 
     for_statement: $ => seq(
@@ -295,22 +318,22 @@ module.exports = grammar({
         $.lexical_declaration,
         $.variable_declaration,
         $.expression_statement,
-        $.empty_statement
+        $.empty_statement,
       )),
       field('condition', choice(
         $.expression_statement,
-        $.empty_statement
+        $.empty_statement,
       )),
       field('increment', optional($._expressions)),
       ')',
-      field('body', $.statement)
+      field('body', $.statement),
     ),
 
     for_in_statement: $ => seq(
       'for',
       optional('await'),
       $._for_header,
-      field('body', $.statement)
+      field('body', $.statement),
     ),
 
     _for_header: $ => seq(
@@ -324,17 +347,17 @@ module.exports = grammar({
           field('kind', 'var'),
           field('left', choice(
             $.identifier,
-            $._destructuring_pattern
+            $._destructuring_pattern,
           )),
-          optional($._initializer)
+          optional($._initializer),
         ),
         seq(
           field('kind', choice('let', 'const')),
           field('left', choice(
             $.identifier,
-            $._destructuring_pattern
-          ))
-        )
+            $._destructuring_pattern,
+          )),
+        ),
       ),
       field('operator', choice('in', 'of')),
       field('right', $._expressions),
@@ -344,65 +367,65 @@ module.exports = grammar({
     while_statement: $ => seq(
       'while',
       field('condition', $.parenthesized_expression),
-      field('body', $.statement)
+      field('body', $.statement),
     ),
 
-    do_statement: $ => seq(
+    do_statement: $ => prec.right(seq(
       'do',
       field('body', $.statement),
       'while',
       field('condition', $.parenthesized_expression),
-      $._semicolon
-    ),
+      optional($._semicolon),
+    )),
 
     try_statement: $ => seq(
       'try',
       field('body', $.statement_block),
       optional(field('handler', $.catch_clause)),
-      optional(field('finalizer', $.finally_clause))
+      optional(field('finalizer', $.finally_clause)),
     ),
 
     with_statement: $ => seq(
       'with',
       field('object', $.parenthesized_expression),
-      field('body', $.statement)
+      field('body', $.statement),
     ),
 
     break_statement: $ => seq(
       'break',
       field('label', optional(alias($.identifier, $.statement_identifier))),
-      $._semicolon
+      $._semicolon,
     ),
 
     continue_statement: $ => seq(
       'continue',
       field('label', optional(alias($.identifier, $.statement_identifier))),
-      $._semicolon
+      $._semicolon,
     ),
 
     debugger_statement: $ => seq(
       'debugger',
-      $._semicolon
+      $._semicolon,
     ),
 
     return_statement: $ => seq(
       'return',
       optional($._expressions),
-      $._semicolon
+      $._semicolon,
     ),
 
     throw_statement: $ => seq(
       'throw',
       $._expressions,
-      $._semicolon
+      $._semicolon,
     ),
 
-    empty_statement: $ => ';',
+    empty_statement: _ => ';',
 
     labeled_statement: $ => prec.dynamic(-1, seq(
       field('label', alias(choice($.identifier, $._reserved_identifier), $.statement_identifier)),
       ':',
-      field('body', $.statement)
+      field('body', $.statement),
     )),
 
     //
@@ -412,37 +435,37 @@ module.exports = grammar({
     switch_body: $ => seq(
       '{',
       repeat(choice($.switch_case, $.switch_default)),
-      '}'
+      '}',
     ),
 
     switch_case: $ => seq(
       'case',
       field('value', $._expressions),
       ':',
-      field('body', repeat($.statement))
+      field('body', repeat($.statement)),
     ),
 
     switch_default: $ => seq(
       'default',
       ':',
-      field('body', repeat($.statement))
+      field('body', repeat($.statement)),
     ),
 
     catch_clause: $ => seq(
       'catch',
       optional(seq('(', field('parameter', choice($.identifier, $._destructuring_pattern)), ')')),
-      field('body', $.statement_block)
+      field('body', $.statement_block),
     ),
 
     finally_clause: $ => seq(
       'finally',
-      field('body', $.statement_block)
+      field('body', $.statement_block),
     ),
 
     parenthesized_expression: $ => seq(
       '(',
       $._expressions,
-      ')'
+      ')',
     ),
 
     //
@@ -450,14 +473,13 @@ module.exports = grammar({
     //
     _expressions: $ => choice(
       $.expression,
-      $.sequence_expression
+      $.sequence_expression,
     ),
 
     expression: $ => choice(
       $.primary_expression,
       $.glimmer_template,
       $._jsx_element,
-      $.jsx_fragment,
       $.assignment_expression,
       $.augmented_assignment_expression,
       $.await_expression,
@@ -484,10 +506,9 @@ module.exports = grammar({
       $.true,
       $.false,
       $.null,
-      $.import,
       $.object,
       $.array,
-      $.function,
+      $.function_expression,
       $.arrow_function,
       $.generator_function,
       $.class,
@@ -499,7 +520,7 @@ module.exports = grammar({
       'yield',
       choice(
         seq('*', $.expression),
-        optional($.expression)
+        optional($.expression),
       ))),
 
     object: $ => prec('object', seq(
@@ -510,10 +531,10 @@ module.exports = grammar({
         $.method_definition,
         alias(
           choice($.identifier, $._reserved_identifier),
-          $.shorthand_property_identifier
-        )
+          $.shorthand_property_identifier,
+        ),
       ))),
-      '}'
+      '}',
     )),
 
     object_pattern: $ => prec('object', seq(
@@ -524,34 +545,34 @@ module.exports = grammar({
         $.object_assignment_pattern,
         alias(
           choice($.identifier, $._reserved_identifier),
-          $.shorthand_property_identifier_pattern
-        )
+          $.shorthand_property_identifier_pattern,
+        ),
       ))),
-      '}'
+      '}',
     )),
 
     assignment_pattern: $ => seq(
       field('left', $.pattern),
       '=',
-      field('right', $.expression)
+      field('right', $.expression),
     ),
 
     object_assignment_pattern: $ => seq(
       field('left', choice(
         alias(choice($._reserved_identifier, $.identifier), $.shorthand_property_identifier_pattern),
-        $._destructuring_pattern
+        $._destructuring_pattern,
       )),
       '=',
-      field('right', $.expression)
+      field('right', $.expression),
     ),
 
     array: $ => seq(
       '[',
       commaSep(optional(choice(
         $.expression,
-        $.spread_element
+        $.spread_element,
       ))),
-      ']'
+      ']',
     ),
 
     array_pattern: $ => seq(
@@ -560,7 +581,7 @@ module.exports = grammar({
         $.pattern,
         $.assignment_pattern,
       ))),
-      ']'
+      ']',
     ),
 
     glimmer_template: $ => choice(
@@ -577,80 +598,87 @@ module.exports = grammar({
       ),
     ),
 
-    _glimmer_template_content: $ => /.{1,}/,
-    glimmer_opening_tag: $ => seq('<template>'),
-    glimmer_closing_tag: $ => seq('</template>'),
+    _glimmer_template_content: _ => /.{1,}/,
+    glimmer_opening_tag: _ => seq('<template>'),
+    glimmer_closing_tag: _ => seq('</template>'),
 
     _jsx_element: $ => choice($.jsx_element, $.jsx_self_closing_element),
 
     jsx_element: $ => seq(
       field('open_tag', $.jsx_opening_element),
       repeat($._jsx_child),
-      field('close_tag', $.jsx_closing_element)
+      field('close_tag', $.jsx_closing_element),
     ),
 
-    jsx_fragment: $ => seq('<', '>', repeat($._jsx_child), '<', '/', '>'),
+    // Should not contain new lines and should not start or end with a space
+    jsx_text: _ => choice(
+      /[^{}<>\n& ]([^{}<>\n&]*[^{}<>\n& ])?/,
+      /\/\/[^\n]*/,
+    ),
 
-    jsx_text: $ => /[^{}<>]+/,
+    // An entity can be named, numeric (decimal), or numeric (hexadecimal). The
+    // longest entity name is 29 characters long, and the HTML spec says that
+    // no more will ever be added.
+    html_character_reference: _ => /&(#([xX][0-9a-fA-F]{1,6}|[0-9]{1,5})|[A-Za-z]{1,30});/,
 
     jsx_expression: $ => seq(
       '{',
       optional(choice(
         $.expression,
         $.sequence_expression,
-        $.spread_element
+        $.spread_element,
       )),
-      '}'
+      '}',
     ),
 
     _jsx_child: $ => choice(
       $.jsx_text,
+      $.html_character_reference,
       $._jsx_element,
-      $.jsx_fragment,
-      $.jsx_expression
+      $.jsx_expression,
     ),
 
     jsx_opening_element: $ => prec.dynamic(-1, seq(
       '<',
-      field('name', $._jsx_element_name),
-      repeat(field('attribute', $._jsx_attribute)),
-      '>'
+      optional(seq(
+        field('name', $._jsx_element_name),
+        repeat(field('attribute', $._jsx_attribute)),
+      )),
+      '>',
     )),
 
-    jsx_identifier: $ => /[a-zA-Z_$][a-zA-Z\d_$]*-[a-zA-Z\d_$\-]*/,
+    jsx_identifier: _ => /[a-zA-Z_$][a-zA-Z\d_$]*-[a-zA-Z\d_$\-]*/,
 
     _jsx_identifier: $ => choice(
       alias($.jsx_identifier, $.identifier),
-      $.identifier
+      $.identifier,
     ),
 
     nested_identifier: $ => prec('member', seq(
-      choice($.identifier, $.nested_identifier),
+      field('object', choice($.identifier, alias($.nested_identifier, $.member_expression))),
       '.',
-      $.identifier
+      field('property', alias($.identifier, $.property_identifier)),
     )),
 
     jsx_namespace_name: $ => seq($._jsx_identifier, ':', $._jsx_identifier),
 
     _jsx_element_name: $ => choice(
       $._jsx_identifier,
-      $.nested_identifier,
+      alias($.nested_identifier, $.member_expression),
       $.jsx_namespace_name,
     ),
 
     jsx_closing_element: $ => seq(
-      '<',
-      '/',
-      field('name', $._jsx_element_name),
-      '>'
+      '</',
+      optional(field('name', $._jsx_element_name)),
+      '>',
     ),
 
     jsx_self_closing_element: $ => seq(
       '<',
       field('name', $._jsx_element_name),
       repeat(field('attribute', $._jsx_attribute)),
-      '/',
-      '>'
+      '/>',
     ),
 
     _jsx_attribute: $ => choice($.jsx_attribute, $.jsx_expression),
@@ -661,15 +689,42 @@ module.exports = grammar({
       $._jsx_attribute_name,
       optional(seq(
         '=',
-        $._jsx_attribute_value
-      ))
+        $._jsx_attribute_value,
+      )),
     ),
 
+    _jsx_string: $ => choice(
+      seq(
+        '"',
+        repeat(choice(
+          alias($.unescaped_double_jsx_string_fragment, $.string_fragment),
+          $.html_character_reference,
+        )),
+        '"',
+      ),
+      seq(
+        '\'',
+        repeat(choice(
+          alias($.unescaped_single_jsx_string_fragment, $.string_fragment),
+          $.html_character_reference,
+        )),
+        '\'',
+      ),
+    ),
+
+    // Workaround to https://github.com/tree-sitter/tree-sitter/issues/1156
+    // We give names to the token() constructs containing a regexp
+    // so as to obtain a node in the CST.
+    //
+    unescaped_double_jsx_string_fragment: _ => token.immediate(prec(1, /[^"&]+/)),
+
+    // same here
+    unescaped_single_jsx_string_fragment: _ => token.immediate(prec(1, /[^'&]+/)),
+
     _jsx_attribute_value: $ => choice(
-      $.string,
+      alias($._jsx_string, $.string),
       $.jsx_expression,
       $._jsx_element,
-      $.jsx_fragment
     ),
 
     class: $ => prec('literal', seq(
@@ -677,7 +732,7 @@ module.exports = grammar({
       'class',
       field('name', optional($.identifier)),
       optional($.class_heritage),
-      field('body', $.class_body)
+      field('body', $.class_body),
     )),
 
     class_declaration: $ => prec('declaration', seq(
@@ -686,17 +741,17 @@ module.exports = grammar({
       field('name', $.identifier),
       optional($.class_heritage),
       field('body', $.class_body),
-      optional($._automatic_semicolon)
+      optional($._automatic_semicolon),
     )),
 
     class_heritage: $ => seq('extends', $.expression),
 
-    function: $ => prec('literal', seq(
+    function_expression: $ => prec('literal', seq(
       optional('async'),
       'function',
       field('name', optional($.identifier)),
       $._call_signature,
-      field('body', $.statement_block)
+      field('body', $.statement_block),
     )),
 
     function_declaration: $ => prec.right('declaration', seq(
@@ -705,7 +760,7 @@ module.exports = grammar({
       field('name', $.identifier),
       $._call_signature,
       field('body', $.statement_block),
-      optional($._automatic_semicolon)
+      optional($._automatic_semicolon),
     )),
 
     generator_function: $ => prec('literal', seq(
@@ -714,7 +769,7 @@ module.exports = grammar({
       '*',
       field('name', optional($.identifier)),
       $._call_signature,
-      field('body', $.statement_block)
+      field('body', $.statement_block),
     )),
 
     generator_function_declaration: $ => prec.right('declaration', seq(
@@ -724,7 +779,7 @@ module.exports = grammar({
       field('name', $.identifier),
       $._call_signature,
       field('body', $.statement_block),
-      optional($._automatic_semicolon)
+      optional($._automatic_semicolon),
     )),
 
     arrow_function: $ => seq(
@@ -734,56 +789,56 @@ module.exports = grammar({
           alias($._reserved_identifier, $.identifier),
           $.identifier,
         )),
-        $._call_signature
+        $._call_signature,
       ),
       '=>',
       field('body', choice(
         $.expression,
-        $.statement_block
-      ))
+        $.statement_block,
+      )),
     ),
 
     // Override
     _call_signature: $ => field('parameters', $.formal_parameters),
     _formal_parameter: $ => choice($.pattern, $.assignment_pattern),
 
-    optional_chain: $ => '?.',
+    optional_chain: _ => '?.',
 
     call_expression: $ => choice(
       prec('call', seq(
-        field('function', $.expression),
-        field('arguments', choice($.arguments, $.template_string))
+        field('function', choice($.expression, $.import)),
+        field('arguments', choice($.arguments, $.template_string)),
       )),
       prec('member', seq(
         field('function', $.primary_expression),
         field('optional_chain', $.optional_chain),
-        field('arguments', $.arguments)
-      ))
+        field('arguments', $.arguments),
+      )),
     ),
 
     new_expression: $ => prec.right('new', seq(
       'new',
       field('constructor', choice($.primary_expression, $.new_expression)),
-      field('arguments', optional(prec.dynamic(1, $.arguments)))
+      field('arguments', optional(prec.dynamic(1, $.arguments))),
     )),
 
     await_expression: $ => prec('unary_void', seq(
       'await',
-      $.expression
+      $.expression,
     )),
 
     member_expression: $ => prec('member', seq(
-      field('object', choice($.expression, $.primary_expression)),
+      field('object', choice($.expression, $.primary_expression, $.import)),
       choice('.', field('optional_chain', $.optional_chain)),
       field('property', choice(
         $.private_property_identifier,
-        alias($.identifier, $.property_identifier)))
+        alias($.identifier, $.property_identifier))),
     )),
 
     subscript_expression: $ => prec.right('member', seq(
       field('object', choice($.expression, $.primary_expression)),
       optional(field('optional_chain', $.optional_chain)),
-      '[', field('index', $._expressions), ']'
+      '[', field('index', $._expressions), ']',
     )),
 
     _lhs_expression: $ => choice(
@@ -791,13 +846,13 @@ module.exports = grammar({
       $.subscript_expression,
       $._identifier,
       alias($._reserved_identifier, $.identifier),
-      $._destructuring_pattern
+      $._destructuring_pattern,
     ),
 
     assignment_expression: $ => prec.right('assign', seq(
       field('left', choice($.parenthesized_expression, $._lhs_expression)),
       '=',
-      field('right', $.expression)
+      field('right', $.expression),
     )),
 
     _augmented_assignment_lhs: $ => choice(
@@ -811,18 +866,18 @@ module.exports = grammar({
     augmented_assignment_expression: $ => prec.right('assign', seq(
       field('left', $._augmented_assignment_lhs),
       field('operator', choice('+=', '-=', '*=', '/=', '%=', '^=', '&=', '|=', '>>=', '>>>=',
-                               '<<=', '**=', '&&=', '||=', '??=')),
-      field('right', $.expression)
+        '<<=', '**=', '&&=', '||=', '??=')),
+      field('right', $.expression),
     )),
 
     _initializer: $ => seq(
       '=',
-      field('value', $.expression)
+      field('value', $.expression),
     ),
 
     _destructuring_pattern: $ => choice(
       $.object_pattern,
-      $.array_pattern
+      $.array_pattern,
     ),
 
     spread_element: $ => seq('...', $.expression),
@@ -832,7 +887,7 @@ module.exports = grammar({
       alias($._ternary_qmark, '?'),
       field('consequence', $.expression),
       ':',
-      field('alternative', $.expression)
+      field('alternative', $.expression),
     )),
 
     binary_expression: $ => choice(
@@ -864,174 +919,154 @@ module.exports = grammar({
         ['in', 'binary_relation'],
       ].map(([operator, precedence, associativity]) =>
         (associativity === 'right' ? prec.right : prec.left)(precedence, seq(
-          field('left', $.expression),
+          field('left', operator === 'in' ? choice($.expression, $.private_property_identifier) : $.expression),
           field('operator', operator),
-          field('right', $.expression)
-        ))
-      )
+          field('right', $.expression),
+        )),
+      ),
     ),
 
     unary_expression: $ => prec.left('unary_void', seq(
       field('operator', choice('!', '~', '-', '+', 'typeof', 'void', 'delete')),
-      field('argument', $.expression)
+      field('argument', $.expression),
     )),
 
     update_expression: $ => prec.left(choice(
       seq(
         field('argument', $.expression),
-        field('operator', choice('++', '--'))
+        field('operator', choice('++', '--')),
       ),
       seq(
         field('operator', choice('++', '--')),
-        field('argument', $.expression)
+        field('argument', $.expression),
       ),
     )),
 
-    sequence_expression: $ => seq(
-      field('left', $.expression),
-      ',',
-      field('right', choice($.sequence_expression, $.expression))
-    ),
+    sequence_expression: $ => prec.right(commaSep1($.expression)),
 
     //
     // Primitives
     //
 
-    // Here we tolerate unescaped newlines in double-quoted and
-    // single-quoted string literals.
-    // This is legal in typescript as jsx/tsx attribute values (as of
-    // 2020), and perhaps will be valid in javascript as well in the
-    // future.
-    //
     string: $ => choice(
       seq(
         '"',
         repeat(choice(
           alias($.unescaped_double_string_fragment, $.string_fragment),
-          $.escape_sequence
+          $.escape_sequence,
         )),
-        '"'
+        '"',
       ),
       seq(
-        "'",
+        '\'',
         repeat(choice(
           alias($.unescaped_single_string_fragment, $.string_fragment),
-          $.escape_sequence
+          $.escape_sequence,
         )),
-        "'"
-      )
+        '\'',
+      ),
     ),
 
     // Workaround to https://github.com/tree-sitter/tree-sitter/issues/1156
     // We give names to the token() constructs containing a regexp
     // so as to obtain a node in the CST.
     //
-    unescaped_double_string_fragment: $ =>
-      token.immediate(prec(1, /[^"\\]+/)),
+    unescaped_double_string_fragment: _ => token.immediate(prec(1, /[^"\\\r\n]+/)),
 
     // same here
-    unescaped_single_string_fragment: $ =>
-      token.immediate(prec(1, /[^'\\]+/)),
+    unescaped_single_string_fragment: _ => token.immediate(prec(1, /[^'\\\r\n]+/)),
 
-    escape_sequence: $ => token.immediate(seq(
+    escape_sequence: _ => token.immediate(seq(
       '\\',
       choice(
         /[^xu0-7]/,
         /[0-7]{1,3}/,
         /x[0-9a-fA-F]{2}/,
         /u[0-9a-fA-F]{4}/,
-        /u{[0-9a-fA-F]+}/
-      )
+        /u{[0-9a-fA-F]+}/,
+        /[\r?][\n\u2028\u2029]/,
+      ),
     )),
 
     // http://stackoverflow.com/questions/13014947/regex-to-match-a-c-style-multiline-comment/36328890#36328890
-    comment: $ => token(choice(
-      seq('//', /.*/),
-      seq(
-        '/*',
-        /[^*]*\*+([^/*][^*]*\*+)*/,
-        '/'
-      ),
-      // https://tc39.es/ecma262/#sec-html-like-comments
-      seq('<!--', /.*/),
-      // This allows code to exist before this token on the same line.
-      //
-      // Technically, --> is supposed to have nothing before it on the same line
-      // except for comments and whitespace, but that is difficult to express,
-      // and in general tree sitter grammars tend to prefer to be overly
-      // permissive anyway.
-      //
-      // This approach does not appear to cause problems in practice.
-      seq('-->', /.*/)
-    )),
+    comment: $ => choice(
+      token(choice(
+        seq('//', /.*/),
+        seq(
+          '/*',
+          /[^*]*\*+([^/*][^*]*\*+)*/,
+          '/',
+        ),
+      )),
+    ),
 
     template_string: $ => seq(
       '`',
       repeat(choice(
-        $._template_chars,
+        alias($._template_chars, $.string_fragment),
         $.escape_sequence,
-        $.template_substitution
+        $.template_substitution,
       )),
-      '`'
+      '`',
     ),
 
     template_substitution: $ => seq(
       '${',
       $._expressions,
-      '}'
+      '}',
     ),
 
     regex: $ => seq(
       '/',
       field('pattern', $.regex_pattern),
-      token.immediate('/'),
-      optional(field('flags', $.regex_flags))
+      token.immediate(prec(1, '/')),
+      optional(field('flags', $.regex_flags)),
     ),
 
-    regex_pattern: $ => token.immediate(prec(-1,
+    regex_pattern: _ => token.immediate(prec(-1,
       repeat1(choice(
         seq(
           '[',
           repeat(choice(
             seq('\\', /./), // escaped character
-            /[^\]\n\\]/       // any character besides ']' or '\n'
+            /[^\]\n\\]/, // any character besides ']' or '\n'
           )),
-          ']'
-        ),              // square-bracket-delimited character class
+          ']',
+        ), // square-bracket-delimited character class
         seq('\\', /./), // escaped character
-        /[^/\\\[\n]/    // any character besides '[', '\', '/', '\n'
-      ))
+        /[^/\\\[\n]/, // any character besides '[', '\', '/', '\n'
+      )),
     )),
 
-    regex_flags: $ => token.immediate(/[a-z]+/),
+    regex_flags: _ => token.immediate(/[a-z]+/),
 
-    number: $ => {
+    number: _ => {
       const hex_literal = seq(
         choice('0x', '0X'),
-        /[\da-fA-F](_?[\da-fA-F])*/
-      )
+        /[\da-fA-F](_?[\da-fA-F])*/,
+      );
 
-      const decimal_digits = /\d(_?\d)*/
-      const signed_integer = seq(optional(choice('-', '+')), decimal_digits)
-      const exponent_part = seq(choice('e', 'E'), signed_integer)
+      const decimal_digits = /\d(_?\d)*/;
+      const signed_integer = seq(optional(choice('-', '+')), decimal_digits);
+      const exponent_part = seq(choice('e', 'E'), signed_integer);
 
-      const binary_literal = seq(choice('0b', '0B'), /[0-1](_?[0-1])*/)
+      const binary_literal = seq(choice('0b', '0B'), /[0-1](_?[0-1])*/);
 
-      const octal_literal = seq(choice('0o', '0O'), /[0-7](_?[0-7])*/)
+      const octal_literal = seq(choice('0o', '0O'), /[0-7](_?[0-7])*/);
 
-      const bigint_literal = seq(choice(hex_literal, binary_literal, octal_literal, decimal_digits), 'n')
+      const bigint_literal = seq(choice(hex_literal, binary_literal, octal_literal, decimal_digits), 'n');
 
       const decimal_integer_literal = choice(
         '0',
-        seq(optional('0'), /[1-9]/, optional(seq(optional('_'), decimal_digits)))
-      )
+        seq(optional('0'), /[1-9]/, optional(seq(optional('_'), decimal_digits))),
+      );
 
       const decimal_literal = choice(
         seq(decimal_integer_literal, '.', optional(decimal_digits), optional(exponent_part)),
         seq('.', decimal_digits, optional(exponent_part)),
         seq(decimal_integer_literal, exponent_part),
         seq(decimal_digits),
-      )
+      );
 
       return token(choice(
         hex_literal,
@@ -1039,7 +1074,7 @@ module.exports = grammar({
         binary_literal,
         octal_literal,
         bigint_literal,
-      ))
+      ));
     },
 
     // 'undefined' is syntactically a regular identifier in JavaScript.
@@ -1049,29 +1084,33 @@ module.exports = grammar({
     // highlight in text editors and other applications.
     _identifier: $ => choice(
       $.undefined,
-      $.identifier
+      $.identifier,
     ),
 
-    identifier: $ => {
-      const alpha = /[^\x00-\x1F\s\p{Zs}0-9:;`"'@#.,|^&<=>+\-*/\\%?!~()\[\]{}\uFEFF\u2060\u200B]|\\u[0-9a-fA-F]{4}|\\u\{[0-9a-fA-F]+\}/
-      const alphanumeric = /[^\x00-\x1F\s\p{Zs}:;`"'@#.,|^&<=>+\-*/\\%?!~()\[\]{}\uFEFF\u2060\u200B]|\\u[0-9a-fA-F]{4}|\\u\{[0-9a-fA-F]+\}/
-      return token(seq(alpha, repeat(alphanumeric)))
+    identifier: _ => {
+      // eslint-disable-next-line max-len
+      const alpha = /[^\x00-\x1F\s\p{Zs}0-9:;`"'@#.,|^&<=>+\-*/\\%?!~()\[\]{}\uFEFF\u2060\u200B]|\\u[0-9a-fA-F]{4}|\\u\{[0-9a-fA-F]+\}/;
+      // eslint-disable-next-line max-len
+      const alphanumeric = /[^\x00-\x1F\s\p{Zs}:;`"'@#.,|^&<=>+\-*/\\%?!~()\[\]{}\uFEFF\u2060\u200B]|\\u[0-9a-fA-F]{4}|\\u\{[0-9a-fA-F]+\}/;
+      return token(seq(alpha, repeat(alphanumeric)));
     },
 
-    private_property_identifier: $ => {
-      const alpha = /[^\x00-\x1F\s\p{Zs}0-9:;`"'@#.,|^&<=>+\-*/\\%?!~()\[\]{}\uFEFF\u2060\u200B]|\\u[0-9a-fA-F]{4}|\\u\{[0-9a-fA-F]+\}/
-      const alphanumeric = /[^\x00-\x1F\s\p{Zs}:;`"'@#.,|^&<=>+\-*/\\%?!~()\[\]{}\uFEFF\u2060\u200B]|\\u[0-9a-fA-F]{4}|\\u\{[0-9a-fA-F]+\}/
-      return token(seq('#', alpha, repeat(alphanumeric)))
+    private_property_identifier: _ => {
+      // eslint-disable-next-line max-len
+      const alpha = /[^\x00-\x1F\s\p{Zs}0-9:;`"'@#.,|^&<=>+\-*/\\%?!~()\[\]{}\uFEFF\u2060\u200B]|\\u[0-9a-fA-F]{4}|\\u\{[0-9a-fA-F]+\}/;
+      // eslint-disable-next-line max-len
+      const alphanumeric = /[^\x00-\x1F\s\p{Zs}:;`"'@#.,|^&<=>+\-*/\\%?!~()\[\]{}\uFEFF\u2060\u200B]|\\u[0-9a-fA-F]{4}|\\u\{[0-9a-fA-F]+\}/;
+      return token(seq('#', alpha, repeat(alphanumeric)));
     },
 
-    meta_property: $ => seq('new', '.', 'target'),
+    meta_property: _ => seq('new', '.', 'target'),
 
-    this: $ => 'this',
-    super: $ => 'super',
-    true: $ => 'true',
-    false: $ => 'false',
-    null: $ => 'null',
-    undefined: $ => 'undefined',
+    this: _ => 'this',
+    super: _ => 'super',
+    true: _ => 'true',
+    false: _ => 'false',
+    null: _ => 'null',
+    undefined: _ => 'undefined',
 
     //
     // Expression components
@@ -1080,7 +1119,7 @@ module.exports = grammar({
     arguments: $ => seq(
       '(',
       commaSep(optional(choice($.expression, $.spread_element))),
-      ')'
+      ')',
     ),
 
     decorator: $ => seq(
@@ -1088,25 +1127,25 @@ module.exports = grammar({
       choice(
         $.identifier,
         alias($.decorator_member_expression, $.member_expression),
-        alias($.decorator_call_expression, $.call_expression)
-      )
+        alias($.decorator_call_expression, $.call_expression),
+      ),
     ),
 
     decorator_member_expression: $ => prec('member', seq(
       field('object', choice(
         $.identifier,
-        alias($.decorator_member_expression, $.member_expression)
+        alias($.decorator_member_expression, $.member_expression),
       )),
       '.',
-      field('property', alias($.identifier, $.property_identifier))
+      field('property', alias($.identifier, $.property_identifier)),
     )),
 
     decorator_call_expression: $ => prec('call', seq(
       field('function', choice(
         $.identifier,
-        alias($.decorator_member_expression, $.member_expression)
+        alias($.decorator_member_expression, $.member_expression),
       )),
-      field('arguments', $.arguments)
+      field('arguments', $.arguments),
     )),
 
     class_body: $ => seq(
@@ -1115,30 +1154,31 @@ module.exports = grammar({
         seq(field('member', $.method_definition), optional(';')),
         seq(field('member', $.field_definition), $._semicolon),
         field('member', $.class_static_block),
-        field('template', $.glimmer_template)
+        field('template', $.glimmer_template),
+        ';',
       )),
-      '}'
+      '}',
     ),
 
     field_definition: $ => seq(
       repeat(field('decorator', $.decorator)),
       optional('static'),
       field('property', $._property_name),
-      optional($._initializer)
+      optional($._initializer),
     ),
 
     formal_parameters: $ => seq(
       '(',
       optional(seq(
         commaSep1($._formal_parameter),
-        optional(',')
+        optional(','),
       )),
-      ')'
+      ')',
     ),
 
     class_static_block: $ => seq(
       'static',
-      field('body', $.statement_block)
+      field('body', $.statement_block),
     ),
 
     // This negative dynamic precedence ensures that during error recovery,
@@ -1146,69 +1186,86 @@ module.exports = grammar({
     // not patterns.
     pattern: $ => prec.dynamic(-1, choice(
       $._lhs_expression,
-      $.rest_pattern
+      $.rest_pattern,
     )),
 
     rest_pattern: $ => prec.right(seq(
       '...',
-      $._lhs_expression
+      $._lhs_expression,
     )),
 
     method_definition: $ => seq(
       repeat(field('decorator', $.decorator)),
-      optional('static'),
+      optional(choice('static', alias(token(seq('static', /\s+/, 'get', /\s*\n/)), 'static get'))),
       optional('async'),
       optional(choice('get', 'set', '*')),
       field('name', $._property_name),
       field('parameters', $.formal_parameters),
-      field('body', $.statement_block)
+      field('body', $.statement_block),
     ),
 
     pair: $ => seq(
       field('key', $._property_name),
       ':',
-      field('value', $.expression)
+      field('value', $.expression),
     ),
 
     pair_pattern: $ => seq(
       field('key', $._property_name),
       ':',
-      field('value', choice($.pattern, $.assignment_pattern))
+      field('value', choice($.pattern, $.assignment_pattern)),
     ),
 
     _property_name: $ => choice(
       alias(choice(
         $.identifier,
-        $._reserved_identifier
+        $._reserved_identifier,
       ), $.property_identifier),
       $.private_property_identifier,
       $.string,
       $.number,
-      $.computed_property_name
+      $.computed_property_name,
     ),
 
     computed_property_name: $ => seq(
       '[',
       $.expression,
-      ']'
+      ']',
     ),
 
-    _reserved_identifier: $ => choice(
+    _reserved_identifier: _ => choice(
       'get',
       'set',
       'async',
       'static',
-      'export'
+      'export',
+      'let',
     ),
 
-    _semicolon: $ => choice($._automatic_semicolon, ';')
-  }
+    _semicolon: $ => choice($._automatic_semicolon, ';'),
+  },
 });
 
+/**
+ * Creates a rule to match one or more of the rules separated by a comma
+ *
+ * @param {Rule} rule
+ *
+ * @return {SeqRule}
+ *
+ */
 function commaSep1(rule) {
   return seq(rule, repeat(seq(',', rule)));
 }
 
+/**
+ * Creates a rule to optionally match one or more of the rules separated by a comma
+ *
+ * @param {Rule} rule
+ *
+ * @return {ChoiceRule}
+ *
+ */
 function commaSep(rule) {
   return optional(commaSep1(rule));
 }
